/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/payment/RetrievePaymentsHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/PaymentApi'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, PaymentApi, CommonUtil, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The RetrievePaymentsHandler model module.
     * @module handler/customer/RetrievePaymentsHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>RetrievePaymentsHandler</code>.
     * @alias module:handler/customer/RetrievePaymentsHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (payments, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(payments);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(payments, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, payments);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (payments) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, payments);

        ValidationUtil.validate(payments[HttpParamConstants.PAGE_SIZE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.PAGE_SIZE, AppErrorCodes.PAGE_SIZE_INVALID, ValidationUtil.isValidPageSize);

        ValidationUtil.validate(payments[HttpParamConstants.QUERY_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(payments[HttpParamConstants.QUERY_PARAM_ID_FUNDING_ACCOUNT], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_PARAM_ID_FUNDING_ACCOUNT, AppErrorCodes.ID_FUNDING_ACCOUNT_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(payments[HttpParamConstants.QUERY_PARAM_ID_CUSTOMER_ACCOUNT], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_PARAM_ID_CUSTOMER_ACCOUNT, AppErrorCodes.ID_CUSTOMER_ACCOUNT_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(payments[HttpParamConstants.CONFIRMATION_NUMBER], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.CONFIRMATION_NUMBER, AppErrorCodes.PAYMENT_CONFIRMATION_NUMBER_INVALID, ValidationUtil.isValidConfirmationNumber);

        ValidationUtil.validate(payments[HttpParamConstants.STATUS], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.STATUS, AppErrorCodes.QUERY_PARAM_PAYMENT_STATUS_INVALID, ValidationUtil.isValidPaymentStatuses);


        ValidationUtil.validate(payments[HttpParamConstants._TXN_PYMNT_SCHLD_TYPE], DataTypeEnum.Array, MandatoryEnum.false, exception,
            HttpParamConstants._TXN_PYMNT_SCHLD_TYPE, AppErrorCodes.TRANSACTION_TYPE_INVALID);


        ValidationUtil.validate(payments[HttpParamConstants.PAYMENT_SCHEDULE_TYPE], DataTypeEnum.Array, MandatoryEnum.false, exception,
            HttpParamConstants.PAYMENT_SCHEDULE_TYPE, AppErrorCodes.PAYMENT_SCHEDULE_TYPE_INVALID, ValidationUtil.isValidPaymentScheduleTypes);

        ValidationUtil.validate(payments[HttpParamConstants.FROM_DATE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.FROM_DATE, AppErrorCodes.PAYMENT_FROM_DATE_INVALID, ValidationUtil.isValidDate);

        ValidationUtil.validate(payments[HttpParamConstants.TO_DATE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.TO_DATE, AppErrorCodes.PAYMENT_TO_DATE_INVALID, ValidationUtil.isValidDate);

        ValidationUtil.validate(payments[HttpParamConstants.QUERY_ID], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_ID, AppErrorCodes.QUERY_ID_INVALID, ValidationUtil.isValidQueryId);

        ValidationUtil.validate(payments[HttpParamConstants.FROM_INDEX], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.FROM_INDEX, AppErrorCodes.FROM_INDEX_INVALID, ValidationUtil.isValidFromIndex);

        ValidationUtil.validate(payments[HttpParamConstants.TO_INDEX], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.TO_INDEX, AppErrorCodes.TO_INDEX_INVALID, ValidationUtil.isValidToIndex);

        ValidationUtil.validateInvocationContext(payments[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(payments[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };

    exports.prototype.getOptions = function (payments) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(payments[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES]),
            'idCustomer': payments[HttpParamConstants.QUERY_PARAM_ID_CUSTOMER],
            'idFundingAccount': payments[HttpParamConstants.QUERY_PARAM_ID_FUNDING_ACCOUNT],
            'idCustomerAccount': payments[HttpParamConstants.QUERY_PARAM_ID_CUSTOMER_ACCOUNT],
            'confirmationNumber': payments[HttpParamConstants.CONFIRMATION_NUMBER],
            'status': payments[HttpParamConstants.STATUS],
            '_txnPymntSchldType': payments[HttpParamConstants._TXN_PYMNT_SCHLD_TYPE],
            'paymentScheduleType': payments[HttpParamConstants.PAYMENT_SCHEDULE_TYPE],
            'fromDate': payments[HttpParamConstants.FROM_DATE],
            'toDate': payments[HttpParamConstants.TO_DATE],
            'pageSize': payments[HttpParamConstants.PAGE_SIZE],
            'queryId': payments[HttpParamConstants.QUERY_ID],
            'fromIndex': payments[HttpParamConstants.FROM_INDEX],
            'toIndex': payments[HttpParamConstants.TO_INDEX]
        };
    };

    exports.prototype.callAPI = function (payments, callback) {
        var _this = this;
        var invocationContext = payments[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var opts = _this.getOptions(payments);
        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], payments[HttpParamConstants.API_END_POINT]);
        var paymentApi = new PaymentApi(apiClient);
        var request = paymentApi.retrievePayments(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType,
            opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));
