/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/base/Base'), require('./InvocationContext'), require('./../../enum/ProductEnum'), require('./../base/Attributes'), require('./../../constant/RestConstants'), require('./../../enum/InvocationModeEnum'), require('./../../util/ValidationUtil'));
    }
}(this, function (logger, InvocationContext, ProductEnum, Attributes, RestConstants, InvocationModeEnum, ValidationUtil) {
    'use strict';


    /**
     * The Base model module.
     * @module model/base/Base
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>Base</code>.
     * @alias module:model/base/Base
     * @class
     */
    var exports = function () {
    };

    /**
     * @private
     * Constructs a <code>Base</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/base/Base} obj Optional instance to populate.
     * @return {module:model/base/Base} The populated <code>Base</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('invocation_context')) {
                obj['invocation_context'] = InvocationContext.constructFromObject(data['invocation_context']);
            }
            if (data.hasOwnProperty('api_end_point')) {
                obj['api_end_point'] = data['api_end_point'];
            }
        }
        return obj;
    };

    /**
     * @member {String} invocation_context
     */
    exports.prototype['invocation_context'] = undefined;
    /**
     * @member {String} api_end_point
     */
    exports.prototype['api_end_point'] = undefined;

    /**
     * @private
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/customer/Customer
     */
    exports.prototype.requestedBy = function (requestor, requestor_type) {
        var _this = this;
        if (!ValidationUtil.isNonEmptyObject(_this['invocation_context'])) {
            _this['invocation_context'] = new InvocationContext();

        }
        _this['invocation_context']['requestor'] = requestor;
        _this['invocation_context']['requestor_type'] = requestor_type;
    };


    /**
     * @param {String} channel
     * @return module:model/customer/Customer
     */
    exports.prototype.videChannel = function (channel) {
        var _this = this;
        if (!ValidationUtil.isNonEmptyObject(_this['invocation_context'])) {
            _this['invocation_context'] = new InvocationContext();

        }
        _this['invocation_context']['channel'] = channel;
    };

    /**
     * @param {String} client_key
     * @return module:model/customer/Customer
     */
    exports.prototype.forClient = function (client_key) {
        var _this = this;
        if (!ValidationUtil.isNonEmptyObject(_this['invocation_context'])) {
            _this['invocation_context'] = new InvocationContext();
        }
        _this['invocation_context']['client_key'] = client_key;
    };

    /**
     * @private
     * @param {String} invocation_mode. Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point. For example http://localhost:8080/payments/v1
     */
    exports.prototype.setAPIEndPoint = function (invocation_mode, api_end_point) {
        var _this = this;
        if (InvocationModeEnum.test === invocation_mode) {
            _this['api_end_point'] = api_end_point;
        } else if (InvocationModeEnum.sandbox === invocation_mode) {
            _this['api_end_point'] = RestConstants.SANDBOX_API_END_POINT;
        } else {
            _this['api_end_point'] = RestConstants.PRODUCTION_API_END_POINT;
        }
    };

    /**
     * @private
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {String} invocation_mode. Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point. For example http://localhost:8080/payments/v1
     * @return module:model/customer/Customer
     */
    exports.prototype.withContext = function (invocation_context, invocation_mode, api_end_point) {
        var _this = this;
        _this.setAPIEndPoint(invocation_mode, api_end_point);
        if (!ValidationUtil.isNonEmptyObject(_this['invocation_context'])) {
            _this['invocation_context'] = new InvocationContext();
        }

        if (invocation_context) {
            _this['invocation_context']['attributes'] = Attributes.constructFromObject(invocation_context['attributes']);
            _this['invocation_context']['idempotent_request_key'] = invocation_context['idempotent_request_key'];
            _this['invocation_context']['product'] = ProductEnum.orbipay_payments;
            _this['invocation_context']['secret'] = invocation_context['secret'];
        }
    };

    return exports;
}));